<?php
/*
Plugin Name: Pajax
Description: Loads a page via ajax.
Version: 1.3.1
Author: Eric Wooley
Author URI: http://www.ericwooley.com/
*/

# get correct id for plugin
$thisfile=basename(__FILE__, ".php");
# register plugin
register_plugin(
	$thisfile, //Plugin id
	'Pajax', 	//Plugin name
	'1.3.1', 		//Plugin version
	'Eric Wooley',  //Plugin author
	'http://www.ericwooley.com/', //author website
	'Handles ajax requests for content.', //Plugin description
	'theme', //page type - on which admin tab to display
	'pajaxSettings'  //main function (administration)
);

# add a link in the admin tab 'theme'
add_action('theme-sidebar','createSideMenu',array($thisfile,'Pajax Settings'));

# Call pajax just before the template loads
add_action('index-pretemplate','pajax');

# Adds the jquery to the theme header
add_action('theme-header', 'pajaxLoaders');


#####################################################################
# Settings Functions
#
# Settings Key
# 0. links to load ajax from
# 1. Head to change.
# 2. Content to change.
# 3. Include jQuery?
# 4. Which animation Effect to use.
# 5. Which loading gif to use.
# 6. weather or not to disable smooth animation css.
# 7. weather the loading bar should be show to the left or right.
#####################################################################


# The path to the pajax settings file. It should be in GSDATAPATH
# but I found that getting permission to write to a file there was 
# inconsistant.
define('PAJAXSETTINGFILE', GSPLUGINPATH."pajax/pajax-settings.txt");

# Allows the user to edit the settings in the admin "theme" section.
function pajaxSettings(){

	# if the user saved settings
	if(isset($_POST["linkDiv"])){
		# creates a string of the new settings dilemeted by "||"
		$newSettings = 	 $_POST["linkDiv"]."||" 			// Setting 0
						.$_POST["headline"]."||"			// Setting 1
						.$_POST["content"]."||" 			// Setting 2
						.$_POST["incJQ"]."||"   			// Setting 3
						.$_POST["effect"]."||"  			// Setting 4
						.$_POST["gif"]."||"     			// Setting 5
						.$_POST["disableSmoothCss"]."||"    // Setting 6
						.$_POST["animationPosition"];		// Setting 7

		# Write the settings to the file.
		if(file_put_contents(PAJAXSETTINGFILE, $newSettings)){
			echo "Settings Updated <br /><br />";
			//print_r($_POST);echo "<br />";print_r(pajaxReadSettings());echo "<br />";
		}
		else
			echo "could not write file: ".PAJAXSETTINGFILE;
	}
	# reload the latest settings
	$settings = pajaxReadSettings();

	# If the "include Jquery" is set then set checked needs to be echoed
	if(strlen($settings[3]) > 1)
		$checked = "checked";
	else
		$checked = "";

	if(strlen($settings[6]) > 1)
		$disableSmoothCss = "checked";
	else
		$disableSmoothCss = "";

	# Echo the form with updated settings
	?>
	<style>
		.pajaxInput{width: 350px;}
	</style>

	<form method='post' action=''>

		<label for="incJQ">Include a copy of jquery? (including multiple jquerys can cause issues in your javascript)</label>
		<input type='checkbox' value='Jq' name="incJQ" id="incJQ" <?php echo $checked ?>/> Include jQuery? <br />
		<br />

		<h3>-- Content Settings --</h3>
		<label for="linkDivs">Jquery object which contains all of your links.</label>
		<input type='text' class='pajaxInput' 
			placeholder="eg: #nav li" value='<?php echo $settings[0]; ?>' name="linkDiv" id="linkDiv" /> <br />
		<br />

		<label for="headline">Jquery object which contains the page title.</label>
		<input type='text' class='pajaxInput' 
			placeholder="eg: #header" value='<?php echo $settings[1]; ?>' name="headline" id="headline" /> <br />
		<br />

		<label for="linkDivs">Jquery object which the new content should be loaded into.</label>
		<input type='text' class='pajaxInput' 
			placeholder="eg: #content" value='<?php echo $settings[2]; ?>' name="content" id="content" /> <br />
		<br />

		<label for="effect">Effect to display before and after loading the new content.</label>
		<select class='pajaxInput' name="effect">
			<?php
				$effects = array("none", "fadeOut / fadeIn", "slideUp / slideDown");
				foreach($effects as $e){
					$sel = "";
					if($e == $settings[4] || ($e == "none" && $settings[4] == ""))
						$sel = "selected";
					?>
					<option <?php echo $sel ?>><?php echo $e ?></option>
					<?php
				}
			?>
		</select>
		<br />
		<input type="checkbox" name="disableSmoothCss" value="disableSmoothCss" <?php echo $disableSmoothCss ?>/>
		Do not include css to smooth animations (check this if it changes something you dont like.)
		<br /><br />
		

		<h3>-- Loading Animations -- <a href="#" id="pajaxHideShow">Show</a></h3>
		<script>
			$(document).ready(function(){
				$("#pajaxHideShow").click(function(){
					$("#pajaxAnimations").slideToggle('slow');
					$(this).html('hide');
					return false;
				});
			});
		</script>
		<div id="pajaxAnimations" style="display:none;">
		<p>If you want a custom gif, upload that gif into 'plugins/pajax/img/' and it will appear in the list.</p>
		Show animation to the <select name="animationPosition">
			<option 
				value="right"
				<?php if($settings[7] == "right") echo "selected"; ?>
			>
				right
			</option>
			<option 
				value="left"
				<?php if($settings[7] == "left") echo "selected"; ?>
			>
				left
			</option>
		</select> of the headline.
		<br /><br />
		<?php
			$dir = GSPLUGINPATH."pajax/img/";
			$gifs = array_diff(scandir($dir), array('.', '..', '.htaccess'));
			$i=0;
			foreach($gifs as $gif){
				$i++;
				if($gif == $settings[5] || ($settings[5] == "" && $i == 1))
					$sel = "checked";
				else
					$sel = "";
				?>
					<input type="radio" name="gif" value="<?php echo $gif ?>" <?php echo $sel ?> />
					<img src='<?php echo $SITEURL.'/plugins/pajax/img/'.$gif ?>' />
					<p><?php echo str_replace(array('-', '.gif'), array(' ', ''), $gif); ?></p>
					<hr />
				<?php
			}
		?>
		</div>

		<input type='submit' value='save settings' />
	</form>

	<?php
}


# Reads the formatted settings from a file and returns an array of the settings
function pajaxReadSettings(){
	if(file_exists(PAJAXSETTINGFILE)){
		$settings = file_get_contents(PAJAXSETTINGFILE);
		return explode("||", $settings);
	}
	return $settings = array();

}

#####################################################################
# Pajax Page Manipulation Functions
#####################################################################

# Sets the theme loaders 
function pajaxLoaders(){
	global $SITEURL;
	#load the pajax settings
	$s = pajaxReadSettings();

	# assign settings to variables
	$nav = $s[0];
	$headline = $s[1];
	$content = $s[2];
	$jq = $s[3];
	$effect = explode(" / ", $s[4]);
	$lGif = $s[5];
	$disableSmoothCss = $s[6];
	$gifPosition = $s[7];

	if(strlen($disableSmoothCss) < 1){
		?>
		<style>
			body{overflow:scroll;}
		</style>
		<?php
	}

	# include jquery if the user wants it.
	if(strlen($jq)>1){
		?>
		
		<script src="https://ajax.googleapis.com/ajax/libs/jquery/1.7.1/jquery.min.js"></script>
		

		<?php
	}

	#echo the jquery to set the onclick methods.
	?>
	<script src="<?php echo $SITEURL.'/plugins/pajax/js/jquery.ba-hashchange.min.js' ?>"></script>
	<script type="text/javascript">
	jQuery(document).ready(function (){


		//#############################################
		// Ajax History
		//#############################################
		// Set the default to determine if this was just loaded and changed.
		window.pajaxLoaded = false;
		// Checks if its the first page.
		window.pajaxFirst = true;

		// Bind the event.
		$(window).hashchange( function(){
			
			// Get the hash and format it.
			var hash = location.hash;
			hash = hash.replace(/\|-\|/g, ' ');
			hash = hash.replace('#', '');
			var first = jQuery('.current a');
			// Abort function if its the first load and the current page matches the hash.
			if(window.pajaxFirst && hash == jQuery('.current a').attr('title'))
				return false;
			
			// If the hash changed but there is none, assume it went back to the first one.
			if(hash.length == 0 && !window.pajaxLoaded && !window.pajaxFirst){
				// The title of the new page being loaded.
				var title = jQuery('.index a').attr("title");
				
				// The adress of the new page being loaded.
				var href = jQuery(".index a").attr("href");
				loadPages(jQuery('.index a'), href, title, false);
			}
			// If there is a hashtag and it was not loaded already
			else if(hash.length > 0 && !window.pajaxLoaded){
				
				// Go through each function in the navigation and check if its the one we need to load.
				jQuery("<?php echo $nav ?>").each(function(){
					var title = $(this).attr('title');
					if(title == hash){
						// The title of the new page being loaded.
						var title = jQuery(this).attr("title");
						
						// The adress of the new page being loaded.
						var href = jQuery(this).attr("href");
						loadPages($(this), href, title, false);
					}
					
				});
			}
			
			window.pajaxLoaded = false;
		});

		// Trigger the event (useful on page load).
		$(window).hashchange();

		//#############################################
		// On a navigation click
		//#############################################
		// binds the onclick to all of the files specified.
		jQuery("<?php echo $nav ?>").click( function(){
			
			// The title of the new page being loaded.
			var title = jQuery(this).attr("title");
			
			// The adress of the new page being loaded.
			var href = jQuery(this).attr("href");
			
			// Change the title to have a loading gif in it.
			var hLoading = 
			<?php 
			if($effect[0] == "none" || $effect[0] == ""){
				?>
				"<img src='<?php echo $SITEURL.'/plugins/pajax/img/'.$lGif ?>' />";
				<?php
			}
			else if($gifPosition == "right"){
				?>
				title + "<img src='<?php echo $SITEURL.'/plugins/pajax/img/'.$lGif ?>' />";
				<?php
			}
			else{
				?>
				"<img src='<?php echo $SITEURL.'/plugins/pajax/img/'.$lGif ?>' /> "+title;
				<?php
			}
			?>
			jQuery("<?php echo $headline ?>").html(hLoading);

			loadPages($(this), href, title, true);
			
			return false;
		});
	});
	function loadPages(currentLink, href, t, setLoad){
			// Reset the current link
			$("<?php echo $nav ?>").parent().removeClass('current');
			currentLink.parent().addClass('current');
			<?php
			$animate = !($effect[0] == "none" || $effect[0] == "");


				// This happens if an effect has been set.
			if($animate){
					?>
					jQuery("<?php echo $content ?>").<?php echo $effect[0] ?>('slow', function(){
				<?php
				}
			?>

			
			// Loading gif in the content area, should only be see if no effect is set.
			var loading = 
			"<center>Loading "+t+"<br /><img src='<?php echo $SITEURL.'/plugins/pajax/img/'.$lGif ?>'</center>";

			// Set the loading animation
			jQuery("<?php echo $content ?>").html(loading);

			// Load the content. with a callback function that will remove all loading indicators and
			// call the final animation, if one is set.
			jQuery("<?php echo $content ?>").load(href, function(){
				<?php
					if($effect[0] != "none" && $effect[0] != ""){
						?>
						jQuery("<?php echo $content ?>").<?php echo $effect[1] ?>('slow');
						<?php
					}
				?>
				// If this was a click this will be true, if loaded from history false
				window.pajaxLoaded = setLoad;

				// If this was from history do not update hash.
				if(setLoad)
					window.location.hash=t.replace(/ /g, '|-|');

				// It is no longer the first call.
				window.pajaxFirst = false;

				// Update the title last to get rid of the gif.
				jQuery("<?php echo $headline ?>").html(t);
			});
			<?php if($animate){
				?>
					});
				<?php
			}
			?>
	}
	</script>
	
	<?php
}

# When the page is an ajax call, only return the page contents and the kill the script.
function pajax(){
	if(isAjax()){
		echo "<!-- Loaded via Pajax -->";
		echo get_page_content();

		// Please let me know if you know of a better way to do this than die().
		die();	
	}
}
# checks if the request is an ajax request.
function isAjax() {
	return (isset($_SERVER['HTTP_X_REQUESTED_WITH']) && 
	($_SERVER['HTTP_X_REQUESTED_WITH'] == 'XMLHttpRequest'));
}